/*++
"This software program is available to you under a choice of one of two
licenses.  You may choose to be licensed under either the GNU General Public
License (GPL) Version 2, June 1991, available at
http://www.fsf.org/copyleft/gpl.html, or the Intel BSD + Patent License,
the text of which follows:

"Recipient" has requested a license and Intel Corporation ("Intel") is willing
to grant a license for the software entitled InfiniBand(tm) System Software
(the "Software") being provided by Intel Corporation.

The following definitions apply to this License:

"Licensed Patents" means patent claims licensable by Intel Corporation which
are necessarily infringed by the use or sale of the Software alone or when
combined with the operating system referred to below.

"Recipient" means the party to whom Intel delivers this Software.
"Licensee" means Recipient and those third parties that receive a license to
any operating system available under the GNU Public License version 2.0 or
later.

Copyright (c) 1996-2003 Intel Corporation. All rights reserved.

The license is provided to Recipient and Recipient's Licensees under the
following terms.

Redistribution and use in source and binary forms of the Software, with or
without modification, are permitted provided that the following conditions are
met:
Redistributions of source code of the Software may retain the above copyright
notice, this list of conditions and the following disclaimer.

Redistributions in binary form of the Software may reproduce the above
copyright notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

Neither the name of Intel Corporation nor the names of its contributors shall
be used to endorse or promote products derived from this Software without
specific prior written permission.

Intel hereby grants Recipient and Licensees a non-exclusive, worldwide,
royalty-free patent license under Licensed Patents to make, use, sell, offer
to sell, import and otherwise transfer the Software, if any, in source code
and object code form. This license shall include changes to the Software that
are error corrections or other minor changes to the Software that do not add
functionality or features when the Software is incorporated in any version of
a operating system that has been distributed under the GNU General Public
License 2.0 or later.  This patent license shall apply to the combination of
the Software and any operating system licensed under the GNU Public License
version 2.0 or later if, at the time Intel provides the Software to Recipient,
such addition of the Software to the then publicly available versions of such
operating system available under the GNU Public License version 2.0 or later
(whether in gold, beta or alpha form) causes such combination to be covered by
the Licensed Patents. The patent license shall not apply to any other
combinations which include the Software. No hardware per se is licensed
hereunder.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL INTEL OR ITS CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
--*/

/*
 * Abstract:
 *	Implementation specific header files for atomic operations.
 *
 * Environment:
 *	Linux Kernel Mode
 *
 * $Revision: 1.2 $
 */

#ifndef _CL_ATOMIC_OSD_H_
#define _CL_ATOMIC_OSD_H_

#include <complib/cl_types.h>

#if defined( CONFIG_X86 )
#include "asm-i386/cl_atomic_asm.h"
#elif defined( CONFIG_X86_64 )
#define LOCK_PREFIX "lock ; "
#include "asm-i386/cl_atomic_asm.h"
#elif defined( CONFIG_PPC )
#include "asm-ppc/cl_atomic_asm.h"
#else				/* not CONFIG_X86 and not CONFIG_PPC */
#include "asm-generic/cl_atomic_asm.h" 
#endif				/* CONFIG_X86 */

static inline int32_t __attribute__ ((unused))
    cl_atomic_inc(IN atomic32_t * const p_value)
{
	return(__cl_atomic_inc(p_value));
}

static inline int32_t __attribute__ ((unused))
    cl_atomic_dec(IN atomic32_t * const p_value)
{
	return(__cl_atomic_dec(p_value));
}

static inline int32_t __attribute__ ((unused))
    cl_atomic_add(IN atomic32_t * const p_value, IN const int32_t increment)
{
	return(__cl_atomic_add(p_value, increment));
}

static inline int32_t __attribute__ ((unused))
    cl_atomic_sub(IN atomic32_t * const p_value, IN const int32_t decrement)
{
	return(__cl_atomic_sub(p_value, decrement));
}

static inline int32_t __attribute__ ((unused))
    cl_atomic_xchg(IN atomic32_t * const p_value, IN const int32_t new_value)
{
#ifdef LINUX
	return (__xchg(new_value, (atomic32_t *) p_value, 4));
#else
   return (__cl_atomic_xchg((atomic32_t *) p_value, new_value));
#endif
}

static inline int32_t __attribute__ ((unused))
    cl_atomic_comp_xchg(IN atomic32_t * const p_value,
		    IN const int32_t compare, IN const int32_t new_value)
{
#ifdef LINUX
	return (cmpxchg(p_value, compare, new_value));
#else
   return (__cl_atomic_cmpxchg(p_value, compare, new_value));
#endif
}

#endif				/* _CL_ATOMIC_OSD_H_ */
