/*++
"This software program is available to you under a choice of one of two
licenses.  You may choose to be licensed under either the GNU General Public
License (GPL) Version 2, June 1991, available at
http://www.fsf.org/copyleft/gpl.html, or the Intel BSD + Patent License,
the text of which follows:

"Recipient" has requested a license and Intel Corporation ("Intel") is willing
to grant a license for the software entitled InfiniBand(tm) System Software
(the "Software") being provided by Intel Corporation.

The following definitions apply to this License:

"Licensed Patents" means patent claims licensable by Intel Corporation which
are necessarily infringed by the use or sale of the Software alone or when
combined with the operating system referred to below.

"Recipient" means the party to whom Intel delivers this Software.
"Licensee" means Recipient and those third parties that receive a license to
any operating system available under the GNU Public License version 2.0 or
later.

Copyright (c) 1996-2003 Intel Corporation. All rights reserved.

The license is provided to Recipient and Recipient's Licensees under the
following terms.

Redistribution and use in source and binary forms of the Software, with or
without modification, are permitted provided that the following conditions are
met:
Redistributions of source code of the Software may retain the above copyright
notice, this list of conditions and the following disclaimer.

Redistributions in binary form of the Software may reproduce the above
copyright notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

Neither the name of Intel Corporation nor the names of its contributors shall
be used to endorse or promote products derived from this Software without
specific prior written permission.

Intel hereby grants Recipient and Licensees a non-exclusive, worldwide,
royalty-free patent license under Licensed Patents to make, use, sell, offer
to sell, import and otherwise transfer the Software, if any, in source code
and object code form. This license shall include changes to the Software that
are error corrections or other minor changes to the Software that do not add
functionality or features when the Software is incorporated in any version of
a operating system that has been distributed under the GNU General Public
License 2.0 or later.  This patent license shall apply to the combination of
the Software and any operating system licensed under the GNU Public License
version 2.0 or later if, at the time Intel provides the Software to Recipient,
such addition of the Software to the then publicly available versions of such
operating system available under the GNU Public License version 2.0 or later
(whether in gold, beta or alpha form) causes such combination to be covered by
the Licensed Patents. The patent license shall not apply to any other
combinations which include the Software. No hardware per se is licensed
hereunder.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL INTEL OR ITS CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
--*/

/*
 * Abstract:
 *	Debug Macros.
 *
 * Environment:
 *	Linux Kernel Mode
 *
 * $Revision: 1.1 $
 */

#ifndef _CL_DEBUG_OSD_H_
#define _CL_DEBUG_OSD_H_

#include <complib/cl_types.h>
#include <complib/cl_timer.h>

#if !defined(__MODULE__)
#define __MODULE__			""
#define __MOD_DELIMITER__	""
#else				/* !defined(__MODULE__) */
#define __MOD_DELIMITER__	":"
#endif				/* !defined(__MODULE__) */

/*
 * Define specifiers for print functions based on the platform
 */
#ifdef __IA64__
#define PRIdSIZE_T	"ld"
#else
#define PRIdSIZE_T	"d"
#endif

#ifdef __KERNEL__

#include <linux/sched.h>
#include <linux/smp.h>
#include <linux/interrupt.h>
#include <asm/types.h>

/* Linux Kernel Mode */
#if BITS_PER_LONG == 64
#define __PRI64_PREFIX	"l"
#else
#define __PRI64_PREFIX	"L"
#endif

#define PRId64		__PRI64_PREFIX"d"
#define PRIo64		__PRI64_PREFIX"o"
#define PRIu64		__PRI64_PREFIX"u"
#define PRIx64		__PRI64_PREFIX"x"

#define cl_msg_out		printk

#if defined (_DEBUG_)
#define cl_dbg_out		printk
#else
#define cl_dbg_out		foo
#endif				/* _DEBUG_ */

/*
 * The following macros are used internally by the CL_ENTER, CL_TRACE,
 * CL_TRACE_EXIT, and CL_EXIT macros.
 */

#if defined (CONFIG_SMP)

#define _CL_DBG_ENTER	\
	("~%d:%s%s%s() [\n", smp_processor_id(), __MODULE__, \
	__MOD_DELIMITER__, __func__)

#define _CL_DBG_EXIT	\
	("~%d:%s%s%s() ]\n", smp_processor_id(), __MODULE__, \
	__MOD_DELIMITER__, __func__)

#define _CL_DBG_INFO	\
	("~%d:%s%s%s(): ", smp_processor_id(), __MODULE__, \
	__MOD_DELIMITER__, __func__)

#define _CL_DBG_ERROR	\
	("~%d:%s%s%s() !ERROR!: ", smp_processor_id(), __MODULE__, \
	__MOD_DELIMITER__, __func__)

#else

#define _CL_DBG_ENTER	\
	("%s%s%s() [\n", __MODULE__, __MOD_DELIMITER__, __func__)

#define _CL_DBG_EXIT	\
	("%s%s%s() ]\n", __MODULE__, __MOD_DELIMITER__, __func__)

#define _CL_DBG_INFO	\
	("%s%s%s(): ", __MODULE__, __MOD_DELIMITER__, __func__)

#define _CL_DBG_ERROR	\
	("%s%s%s() !ERROR!: ", __MODULE__, __MOD_DELIMITER__, __func__)

#endif

#ifdef CONFIG_X86
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
#define CL_STK_LIMIT ((uint32_t)(current->thread_info) + sizeof(struct thread_info))
#else
#define CL_STK_LIMIT ((uint32_t)current + sizeof(struct task_struct))
#endif

#define CL_CHK_STK												\
{																\
	uint32_t __CL_ESP__;										\
	__asm__ __volatile__("movl %%esp,%0" : "=r" (__CL_ESP__));	\
	if ((CL_STK_LIMIT +	                                        \
	    (in_interrupt() ? 300: 1024)) > __CL_ESP__)				\
	{															\
		cl_msg_out("stack corruption detected!!!\n");			\
		cl_msg_out("::::::::esp(0x%x) top(0x%x)::::::::\n",__CL_ESP__,\
			((uint32_t)current + sizeof(struct task_struct) +	\
			(in_interrupt() ? 300: 1024)));						\
		CL_ASSERT (0);											\
	}															\
}
#else
#define CL_CHK_STK		/* We do not do checks for 64 for now... */
#endif

#else				/* __KERNEL__ */

/* Linux User Mode */
#include <inttypes.h>
#include <stdio.h>

#define cl_msg_out	printf

#if defined( _DEBUG_ )
#define cl_dbg_out	printf
#else
#define cl_dbg_out	foo
#endif				/* _DEBUG_ */

/*
 * The following macros are used internally by the CL_ENTER, CL_TRACE,
 * CL_TRACE_EXIT, and CL_EXIT macros.
 */
#define _CL_DBG_ENTER	\
	("%d:%lld:%s%s%s() [\n", getpid(), cl_get_time_stamp(),\
	__MODULE__, __MOD_DELIMITER__, __func__)

#define _CL_DBG_EXIT	\
	("%d:%lld:%s%s%s() ]\n", getpid(), cl_get_time_stamp(),\
	__MODULE__, __MOD_DELIMITER__, __func__)

#define _CL_DBG_INFO	\
	("%d:%lld:%s%s%s(): ", getpid(), cl_get_time_stamp(),\
	__MODULE__, __MOD_DELIMITER__, __func__)

#define _CL_DBG_ERROR	\
	("%d:%lld:%s%s%s() !ERROR!: ", getpid(), cl_get_time_stamp(),\
	__MODULE__, __MOD_DELIMITER__, __func__)

#define CL_CHK_STK

#endif				/* __KERNEL__ */

#endif				/* _CL_DEBUG_OSD_H_ */
