/*++
"This software program is available to you under a choice of one of two
licenses.  You may choose to be licensed under either the GNU General Public
License (GPL) Version 2, June 1991, available at
http://www.fsf.org/copyleft/gpl.html, or the Intel BSD + Patent License,
the text of which follows:

"Recipient" has requested a license and Intel Corporation ("Intel") is willing
to grant a license for the software entitled InfiniBand(tm) System Software
(the "Software") being provided by Intel Corporation.

The following definitions apply to this License:

"Licensed Patents" means patent claims licensable by Intel Corporation which
are necessarily infringed by the use or sale of the Software alone or when
combined with the operating system referred to below.

"Recipient" means the party to whom Intel delivers this Software.
"Licensee" means Recipient and those third parties that receive a license to
any operating system available under the GNU Public License version 2.0 or
later.

Copyright (c) 1996-2003 Intel Corporation. All rights reserved.

The license is provided to Recipient and Recipient's Licensees under the
following terms.

Redistribution and use in source and binary forms of the Software, with or
without modification, are permitted provided that the following conditions are
met:
Redistributions of source code of the Software may retain the above copyright
notice, this list of conditions and the following disclaimer.

Redistributions in binary form of the Software may reproduce the above
copyright notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

Neither the name of Intel Corporation nor the names of its contributors shall
be used to endorse or promote products derived from this Software without
specific prior written permission.

Intel hereby grants Recipient and Licensees a non-exclusive, worldwide,
royalty-free patent license under Licensed Patents to make, use, sell, offer
to sell, import and otherwise transfer the Software, if any, in source code
and object code form. This license shall include changes to the Software that
are error corrections or other minor changes to the Software that do not add
functionality or features when the Software is incorporated in any version of
a operating system that has been distributed under the GNU General Public
License 2.0 or later.  This patent license shall apply to the combination of
the Software and any operating system licensed under the GNU Public License
version 2.0 or later if, at the time Intel provides the Software to Recipient,
such addition of the Software to the then publicly available versions of such
operating system available under the GNU Public License version 2.0 or later
(whether in gold, beta or alpha form) causes such combination to be covered by
the Licensed Patents. The patent license shall not apply to any other
combinations which include the Software. No hardware per se is licensed
hereunder.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL INTEL OR ITS CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
--*/

/*
 * Abstract:
 *	This header file defines data structures and APIs for the system helper
 *	module of the component library.
 *
 * Environment:
 *	Linux Kernel and User Mode.
 *
 * $Revision: 1.1 $
 */

#ifndef _CL_SYSHELPER_H_
#define _CL_SYSHELPER_H_

/****h* Component Library/System Helper
* NAME
*	System Helper
*
* DESCRIPTION
*	Provides ioctl support and handle validation for wait objects.
*
********/

#ifdef	__KERNEL__

#include <linux/ioctl.h>
#include <linux/mm.h>

#endif				//__KERNEL__

#include <complib/cl_types.h>
#include <complib/cl_list.h>
#include <complib/cl_qlist.h>
#include <complib/cl_event.h>
#include <complib/cl_qmap.h>
#include <complib/cl_passivelock.h>
#include <complib/cl_spinlock.h>

#define SYSHELP_DEVICE_NAME	"/dev/cl_dev"
#define	SYSDEV_KEY	'#'

#ifdef	__KERNEL__

/****f* Component Library: System Helper/__cl_syshelper_init
* NAME
*	__cl_syshelper_init
*
* DESCRIPTION
*	Initializes the system helper data structures and prepares it
*	for use.
*
* SYNOPSIS
*/
cl_status_t __cl_syshelper_init(void);

/*
* PARAMETERS
*	None.
*
* RETURN VALUES
*	CL_SUCCESS
*		The initialization completed successfully.
*	CL_ERROR
*		Could not create the device.
*
* NOTES
*
* SEE ALSO
*	__cl_syshelper_exit
********/

/****f* Component Library: System Helper/__cl_syshelper_exit
* NAME
*	__cl_syshelper_exit
*
* DESCRIPTION
*	Releases the resources used by syshelper and destroys the device.
*
* SYNOPSIS
*/
void
 __cl_syshelper_exit(void);

/*
* PARAMETERS
*	None
*
* RETURN VALUES
*	None.
*
* NOTES
*
* SEE ALSO
*	__cl_syshelper_init
*********/

#else				// __KERNEL__

/*
 * User mode only
 */

/****f* Component Library: System Helper/__cl_user_syshelper_init
* NAME
*	__cl_user_syshelper_init
*
* DESCRIPTION
*	Initialize the system helper in user mode.
*
* SYNOPSIS
*/
cl_status_t __cl_user_syshelper_init(void);
/*
* PARAMETERS
*	None.
*
* RETURN VALUES
*	CL_SUCCESS
*		The initialization completed successfully.
*
* NOTES
*
* SEE ALSO
*****/

/****f* Component Library: System Helper/__cl_user_syshelper_exit
* NAME
*	__cl_user_syshelper_exit
*
* DESCRIPTION
*	Cleanup the system helper in user mode.
*
* SYNOPSIS
*/
void
 __cl_user_syshelper_exit(void);
/*
* PARAMETERS
*	None.
*
* RETURN VALUES
*	None.
*
* NOTES
*
* SEE ALSO
******/

#endif				// __KERNEL__

/*
 * Shared between user and kernel mode.
 */

/****d* Component Library: System Helper/cl_syshelper_ops_t
* NAME
*	syshelper_ops_t
*
* DESCRIPTION
*
* SYNOPSIS
*/
typedef enum cl_syshelper_ops {
	create_wait_obj = 1,
	waiton_wait_obj,
	trigger_wait_obj,
	reset_wait_obj,
	syshelp_ioctl_max	/* always at the end of the list */
} cl_syshelper_ops_t;
/**********/

/*
 * Various Opration Allowable on the System Helper
 */
#define	CREATE_WAIT_OBJ		\
					IOCTL_CMD(SYSDEV_KEY, create_wait_obj)
#define	WAITON_WAIT_OBJ		\
					IOCTL_CMD(SYSDEV_KEY, waiton_wait_obj)
#define	TRIGGER_WAIT_OBJ	\
					IOCTL_CMD(SYSDEV_KEY, trigger_wait_obj)
#define	RESET_WAIT_OBJ		\
					IOCTL_CMD(SYSDEV_KEY, reset_wait_obj)

#endif				//_CL_SYSHELPER_H_
