/*++
"This software program is available to you under a choice of one of two
licenses.  You may choose to be licensed under either the GNU General Public
License (GPL) Version 2, June 1991, available at
http://www.fsf.org/copyleft/gpl.html, or the Intel BSD + Patent License,
the text of which follows:

"Recipient" has requested a license and Intel Corporation ("Intel") is willing
to grant a license for the software entitled InfiniBand(tm) System Software
(the "Software") being provided by Intel Corporation.

The following definitions apply to this License:

"Licensed Patents" means patent claims licensable by Intel Corporation which
are necessarily infringed by the use or sale of the Software alone or when
combined with the operating system referred to below.

"Recipient" means the party to whom Intel delivers this Software.
"Licensee" means Recipient and those third parties that receive a license to
any operating system available under the GNU Public License version 2.0 or
later.

Copyright (c) 1996-2003 Intel Corporation. All rights reserved.

The license is provided to Recipient and Recipient's Licensees under the
following terms.

Redistribution and use in source and binary forms of the Software, with or
without modification, are permitted provided that the following conditions are
met:
Redistributions of source code of the Software may retain the above copyright
notice, this list of conditions and the following disclaimer.

Redistributions in binary form of the Software may reproduce the above
copyright notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

Neither the name of Intel Corporation nor the names of its contributors shall
be used to endorse or promote products derived from this Software without
specific prior written permission.

Intel hereby grants Recipient and Licensees a non-exclusive, worldwide,
royalty-free patent license under Licensed Patents to make, use, sell, offer
to sell, import and otherwise transfer the Software, if any, in source code
and object code form. This license shall include changes to the Software that
are error corrections or other minor changes to the Software that do not add
functionality or features when the Software is incorporated in any version of
a operating system that has been distributed under the GNU General Public
License 2.0 or later.  This patent license shall apply to the combination of
the Software and any operating system licensed under the GNU Public License
version 2.0 or later if, at the time Intel provides the Software to Recipient,
such addition of the Software to the then publicly available versions of such
operating system available under the GNU Public License version 2.0 or later
(whether in gold, beta or alpha form) causes such combination to be covered by
the Licensed Patents. The patent license shall not apply to any other
combinations which include the Software. No hardware per se is licensed
hereunder.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL INTEL OR ITS CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
--*/

/*
 * Abstract:
 *	Defines sized datatypes for Linux Kernel and User mode
 *  exported sizes are int8_t, uint8_t, int16_t, uint16_t, int32_t, uint32_t
 *  int64_t, uint64_t. uintn_t is a polymorphic type, size is native size and
 *  also size of the pointer.
 *
 * Environment:
 *	Linux User and Kernel Mode
 *
 * $Revision: 1.1 $
 */

#ifndef _CL_TYPES_OSD_H_
#define _CL_TYPES_OSD_H_

#if defined (_DEBUG_)
#ifdef __IA64__
#define cl_break() asm("   break 0")
#else				/* __IA64__ */
#define cl_break() asm("   int $3")
#endif				/* __IA64__ */
#else				/* _DEBUG_ */
#define cl_break
#endif

#ifdef __KERNEL__
#include <linux/config.h>

/*
 * Linux Kernel Mode
 */

#if defined (_DEBUG_) && \
	(defined (CONFIG_X86_REMOTE_DEBUG) || defined(CONFIG_KGDB))
#define CONFIG_GDB
#endif

#if defined(CONFIG_MODVERSIONS) && !defined(MODVERSIONS)
#define MODVERSIONS		/* turn it on */
#endif

#ifndef LINUX_VERSION_CODE
#include <linux/version.h>
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif
#endif

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/string.h>
#include <linux/init.h>

#if defined (_DEBUG_)

#if defined (CONFIG_GDB)

#define CL_ASSERT( __exp__ )										\
{																\
	if( !(__exp__) )											\
	{															\
			printk( "Assertion failed: %s, file %s, line %d\n",	\
					#__exp__, __FILE__, __LINE__ );				\
			printk ("Entering GDB...\n");						\
			cl_break();											\
	}															\
}
#elif defined (CONFIG_KDB)	/* not CONFIG_GDB */
#include <linux/kdb.h>
#undef cl_break
#define cl_break()	KDB_ENTER()

#define CL_ASSERT( __exp__ )										\
{																\
	if( !(__exp__) )											\
	{															\
			printk( "Assertion failed: %s, file %s, line %d\n",	\
					#__exp__, __FILE__, __LINE__ );				\
			printk ("Entering KDB...\n");						\
			cl_break();											\
	}															\
}
#else				/* not CONFIG_GDB and not CONFIG_KDB */
#undef cl_break
#define cl_break()

#define CL_ASSERT( __exp__ )									\
{															\
	if( !(__exp__) )										\
	{														\
		panic( "Assertion failed: %s, file %s, line %d\n",	\
				#__exp__, __FILE__, __LINE__ );				\
	}														\
}
#endif				/* CONFIG_GDB */

#else				/* not _DEBUG_ */
#undef cl_break
#define cl_break()
#define CL_ASSERT( __exp__ )
#endif				/* _DEBUG_ */

#define cl_panic	panic

#else				/* __KERNEL__ */

/*
 * Linux User Mode
 */
#include <inttypes.h>
#include <assert.h>
#include <string.h>

#if defined (_DEBUG_)
#define CL_ASSERT	assert
#else				/* _DEBUG_ */
#define CL_ASSERT( __exp__ )
#endif				/* _DEBUG_ */

#endif				/* __KERNEL__ */

/*
 * Types not explicitly defined are native to the platform.
 */
typedef unsigned long uintn_t;
typedef long intn_t;
typedef int boolean_t;
typedef volatile int32_t atomic32_t;

#ifndef NULL
#define NULL	(void*)0
#endif

#define UNUSED_PARAM( P )

#endif				/* _CL_TYPES_OSD_H_ */
