/*++
"This software program is available to you under a choice of one of two
licenses.  You may choose to be licensed under either the GNU General Public
License (GPL) Version 2, June 1991, available at
http://www.fsf.org/copyleft/gpl.html, or the Intel BSD + Patent License,
the text of which follows:

"Recipient" has requested a license and Intel Corporation ("Intel") is willing
to grant a license for the software entitled InfiniBand(tm) System Software
(the "Software") being provided by Intel Corporation.

The following definitions apply to this License:

"Licensed Patents" means patent claims licensable by Intel Corporation which
are necessarily infringed by the use or sale of the Software alone or when
combined with the operating system referred to below.

"Recipient" means the party to whom Intel delivers this Software.
"Licensee" means Recipient and those third parties that receive a license to
any operating system available under the GNU Public License version 2.0 or
later.

Copyright (c) 1996-2003 Intel Corporation. All rights reserved.

The license is provided to Recipient and Recipient's Licensees under the
following terms.

Redistribution and use in source and binary forms of the Software, with or
without modification, are permitted provided that the following conditions are
met:
Redistributions of source code of the Software may retain the above copyright
notice, this list of conditions and the following disclaimer.

Redistributions in binary form of the Software may reproduce the above
copyright notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

Neither the name of Intel Corporation nor the names of its contributors shall
be used to endorse or promote products derived from this Software without
specific prior written permission.

Intel hereby grants Recipient and Licensees a non-exclusive, worldwide,
royalty-free patent license under Licensed Patents to make, use, sell, offer
to sell, import and otherwise transfer the Software, if any, in source code
and object code form. This license shall include changes to the Software that
are error corrections or other minor changes to the Software that do not add
functionality or features when the Software is incorporated in any version of
a operating system that has been distributed under the GNU General Public
License 2.0 or later.  This patent license shall apply to the combination of
the Software and any operating system licensed under the GNU Public License
version 2.0 or later if, at the time Intel provides the Software to Recipient,
such addition of the Software to the then publicly available versions of such
operating system available under the GNU Public License version 2.0 or later
(whether in gold, beta or alpha form) causes such combination to be covered by
the Licensed Patents. The patent license shall not apply to any other
combinations which include the Software. No hardware per se is licensed
hereunder.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL INTEL OR ITS CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
--*/

/*
 * Abstract:
 *	One stop shopping for component library headers.
 *
 * Environment:
 *	All
 *
 * $Revision: 1.1 $
 */

#ifndef _CL_LIB_H_
#define _CL_LIB_H_

/****h* Component Library/Overview
* NAME
*	Overview
*
* DESCRIPTION
*	The component library is a collection of components that can be used to
*	create complex projects quickly and reliably.
*
*	The component library simplifies development by eliminating the need to
*	re-implement existing functionality. This contributes to shorter
*	development cycles as well as smaller code bases, helping reduce the
*	number of bugs by leveraging tried and tested code.
*
*	The component library also provides the same interface in multiple
*	environments, such as kernel mode and user mode, allowing code to be used
*	in both, again reducing code duplication and development life cycles.
*
*	Components of the library all follow the same usage model, as follows:
*		- The constructor for all components must be called before any other
*		function for that component.
*
*		- The initializer for all components must be called successfully
*		before any function manipulating that component is called.
*
*		- The destructor for all components must be called if the constructor
*		was called.
*
*	In a debug build, the components assert that the proper sequence is
*	followed.
*********/

#include <complib/cl_types.h>
#include <complib/cl_debug.h>
#include <complib/cl_memory.h>
#include <complib/cl_atomic.h>
#include <complib/cl_byteswap.h>
#include <complib/cl_math.h>
#include <complib/cl_passivelock.h>
#include <complib/cl_spinlock.h>
#include <complib/cl_timer.h>
#include <complib/cl_event.h>
#include <complib/cl_qlist.h>
#include <complib/cl_list.h>
#include <complib/cl_qcomppool.h>
#include <complib/cl_qpool.h>
#include <complib/cl_comppool.h>
#include <complib/cl_pool.h>
#include <complib/cl_reqmgr.h>
#include <complib/cl_vector.h>
#include <complib/cl_syscallback.h>
#include <complib/cl_thread.h>
#include <complib/cl_threadpool.h>
#include <complib/cl_perf.h>
#include <complib/cl_log.h>
#include <complib/cl_qmap.h>
#include <complib/cl_map.h>
#include <complib/cl_fleximap.h>
#include <complib/cl_async_proc.h>
#include <complib/cl_ptr_vector.h>
#include <complib/cl_qlockpool.h>
#include <complib/cl_mutex.h>
#include <complib/cl_obj.h>

#endif				/* _CL_LIB_H_ */
