/*++
"This software program is available to you under a choice of one of two
licenses.  You may choose to be licensed under either the GNU General Public
License (GPL) Version 2, June 1991, available at
http://www.fsf.org/copyleft/gpl.html, or the Intel BSD + Patent License,
the text of which follows:

"Recipient" has requested a license and Intel Corporation ("Intel") is willing
to grant a license for the software entitled InfiniBand(tm) System Software
(the "Software") being provided by Intel Corporation.

The following definitions apply to this License:

"Licensed Patents" means patent claims licensable by Intel Corporation which
are necessarily infringed by the use or sale of the Software alone or when
combined with the operating system referred to below.

"Recipient" means the party to whom Intel delivers this Software.
"Licensee" means Recipient and those third parties that receive a license to
any operating system available under the GNU Public License version 2.0 or
later.

Copyright (c) 1996-2003 Intel Corporation. All rights reserved.

The license is provided to Recipient and Recipient's Licensees under the
following terms.

Redistribution and use in source and binary forms of the Software, with or
without modification, are permitted provided that the following conditions are
met:
Redistributions of source code of the Software may retain the above copyright
notice, this list of conditions and the following disclaimer.

Redistributions in binary form of the Software may reproduce the above
copyright notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

Neither the name of Intel Corporation nor the names of its contributors shall
be used to endorse or promote products derived from this Software without
specific prior written permission.

Intel hereby grants Recipient and Licensees a non-exclusive, worldwide,
royalty-free patent license under Licensed Patents to make, use, sell, offer
to sell, import and otherwise transfer the Software, if any, in source code
and object code form. This license shall include changes to the Software that
are error corrections or other minor changes to the Software that do not add
functionality or features when the Software is incorporated in any version of
a operating system that has been distributed under the GNU General Public
License 2.0 or later.  This patent license shall apply to the combination of
the Software and any operating system licensed under the GNU Public License
version 2.0 or later if, at the time Intel provides the Software to Recipient,
such addition of the Software to the then publicly available versions of such
operating system available under the GNU Public License version 2.0 or later
(whether in gold, beta or alpha form) causes such combination to be covered by
the Licensed Patents. The patent license shall not apply to any other
combinations which include the Software. No hardware per se is licensed
hereunder.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL INTEL OR ITS CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
--*/

/*
 * Abstract:
 *	This module defines
 *
 * Environment:
 *	Linux Kernel Mode
 *
 * $Revision: 1.1 $
 */
#include <complib/cl_types.h>
#include <complib/cl_memory.h>
#include <complib/cl_debug.h>
#include <complib/cl_list.h>
#include <complib/cl_event.h>
#include <complib/cl_qmap.h>
#include <complib/cl_syshelper.h>

/****i* Component Library Wait Objects/k_wait_object_t
* NAME
*	k_wait_object_t
*
* PURPOSE
*	Kernel wait object representation.
*
* SYNOPSIS
*
******/

typedef struct _cl_wait_object {
	cl_event_t event;
	void *p_owner;

} cl_wait_object_t;
/********/

#include <complib/cl_waitobj.h>

/*
* user-mode or kernel-mode clients call this to create wait object
*/
cl_status_t
__cl_create_wait_object(IN boolean_t auto_reset,
			OUT cl_wait_obj_handle_t * p_wait_obj_handle)
{
	cl_wait_object_t *p_k_wait_obj = NULL;

	/* Allocate memory for helper wait object */
	p_k_wait_obj =
	    (cl_wait_object_t *) cl_zalloc(sizeof (cl_wait_object_t));
	if (p_k_wait_obj == NULL) {
		return CL_INSUFFICIENT_MEMORY;
	}

	/* construct and initialize the event */
	cl_event_construct(&p_k_wait_obj->event);
	cl_event_init(&p_k_wait_obj->event, !auto_reset);

	/* Set the owner of this wait object */
	p_k_wait_obj->p_owner = current;

	/* Return the handle */
	*p_wait_obj_handle = (cl_wait_obj_handle_t) (p_k_wait_obj);

	return CL_SUCCESS;
}

/*
 * This function is used by kernel clients only
 */
cl_status_t
cl_create_wait_object(IN boolean_t auto_reset,
		      OUT cl_wait_obj_handle_t * p_wait_obj_handle)
{
	return __cl_create_wait_object(auto_reset, p_wait_obj_handle);
}

/*
 * user-mode or kernel-mode clients call this to wait on a wait object
 */
cl_status_t
__cl_wait_on_wait_object(IN cl_wait_obj_handle_t wait_obj_handle,
			 IN uint32_t wait_u_sec)
{
	cl_status_t status = CL_SUCCESS;
	cl_wait_object_t *p_k_wait_obj = NULL;

	p_k_wait_obj = (cl_wait_object_t *) wait_obj_handle;

#ifdef NON_INTERRUPTIBLE
	status = cl_event_wait_on(&p_k_wait_obj->event, wait_u_sec, FALSE);
#else
	status = cl_event_wait_on(&p_k_wait_obj->event, wait_u_sec, TRUE);
#endif

	return status;
}

/*
 * This function is used by kernel clients only
 */
cl_status_t
cl_wait_on_wait_object(IN cl_wait_obj_handle_t wait_obj_handle,
		       IN uint32_t wait_u_sec)
{
	return __cl_wait_on_wait_object(wait_obj_handle, wait_u_sec);
}

/*
 * user-mode or kernel-mode clients call this to trigger a wait object
 */
cl_status_t
__cl_signal_wait_object(IN cl_wait_obj_handle_t wait_obj_handle)
{
	cl_wait_object_t *p_k_wait_obj = NULL;

	p_k_wait_obj = (cl_wait_object_t *) wait_obj_handle;

	cl_event_signal(&p_k_wait_obj->event);

	return CL_SUCCESS;
}

/*
 * This function is used by kernel clients only
 */
cl_status_t
cl_signal_wait_object(IN cl_wait_obj_handle_t wait_obj_handle)
{
	return __cl_signal_wait_object(wait_obj_handle);
}

/*
 * user-mode or kernel-mode clients call this to trigger a wait object
 */
cl_status_t
__cl_clear_wait_object(IN cl_wait_obj_handle_t wait_obj_handle)
{
	cl_wait_object_t *p_k_wait_obj = NULL;

	p_k_wait_obj = (cl_wait_object_t *) wait_obj_handle;

	return (cl_event_reset(&p_k_wait_obj->event));
}

/*
 * This function is used by kernel clients only
 */
cl_status_t
cl_clear_wait_object(IN cl_wait_obj_handle_t wait_obj_handle)
{
	return __cl_clear_wait_object(wait_obj_handle);
}

/*
 * user-mode or kernel-mode clients call this to trigger a wait object
 */
cl_status_t
__cl_destroy_wait_object(IN cl_wait_obj_handle_t wait_obj_handle)
{
	cl_wait_object_t *p_k_wait_obj = NULL;

	p_k_wait_obj = (cl_wait_object_t *) wait_obj_handle;

	cl_event_destroy(&p_k_wait_obj->event);

	cl_free(p_k_wait_obj);

	return CL_SUCCESS;
}

/*
 * This function is used by kernel clients only
 */
cl_status_t
cl_destroy_wait_object(IN cl_wait_obj_handle_t wait_obj_handle)
{
	return __cl_destroy_wait_object(wait_obj_handle);
}
