/*++
"This software program is available to you under a choice of one of two
licenses.  You may choose to be licensed under either the GNU General Public
License (GPL) Version 2, June 1991, available at
http://www.fsf.org/copyleft/gpl.html, or the Intel BSD + Patent License,
the text of which follows:

"Recipient" has requested a license and Intel Corporation ("Intel") is willing
to grant a license for the software entitled InfiniBand(tm) System Software
(the "Software") being provided by Intel Corporation.

The following definitions apply to this License:

"Licensed Patents" means patent claims licensable by Intel Corporation which
are necessarily infringed by the use or sale of the Software alone or when
combined with the operating system referred to below.

"Recipient" means the party to whom Intel delivers this Software.
"Licensee" means Recipient and those third parties that receive a license to
any operating system available under the GNU Public License version 2.0 or
later.

Copyright (c) 1996-2003 Intel Corporation. All rights reserved.

The license is provided to Recipient and Recipient's Licensees under the
following terms.

Redistribution and use in source and binary forms of the Software, with or
without modification, are permitted provided that the following conditions are
met:
Redistributions of source code of the Software may retain the above copyright
notice, this list of conditions and the following disclaimer.

Redistributions in binary form of the Software may reproduce the above
copyright notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

Neither the name of Intel Corporation nor the names of its contributors shall
be used to endorse or promote products derived from this Software without
specific prior written permission.

Intel hereby grants Recipient and Licensees a non-exclusive, worldwide,
royalty-free patent license under Licensed Patents to make, use, sell, offer
to sell, import and otherwise transfer the Software, if any, in source code
and object code form. This license shall include changes to the Software that
are error corrections or other minor changes to the Software that do not add
functionality or features when the Software is incorporated in any version of
a operating system that has been distributed under the GNU General Public
License 2.0 or later.  This patent license shall apply to the combination of
the Software and any operating system licensed under the GNU Public License
version 2.0 or later if, at the time Intel provides the Software to Recipient,
such addition of the Software to the then publicly available versions of such
operating system available under the GNU Public License version 2.0 or later
(whether in gold, beta or alpha form) causes such combination to be covered by
the Licensed Patents. The patent license shall not apply to any other
combinations which include the Software. No hardware per se is licensed
hereunder.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL INTEL OR ITS CONTRIBUTORS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
--*/

/*
 * Standard user mode includes
 */

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <string.h>

#include <complib/cl_types.h>
#include <complib/cl_debug.h>
#include <complib/cl_device.h>


cl_status_t
cl_open_device(
	IN cl_dev_name_t	device_name,
	IN cl_dev_handle_t	*p_dev_handle )
{
	/* sanity check */
	if ( p_dev_handle == NULL)
	{
		return CL_INVALID_PARAMETER;
	}
	*p_dev_handle = open(device_name, O_RDWR);

	if (*p_dev_handle < 0)
	{
		*p_dev_handle = 0;
		return CL_ERROR;
	}
	else
	{
		return CL_SUCCESS;
	}
}

void
cl_close_device(
	IN cl_dev_handle_t dev_handle )
{
	int	status = 0;

	status = close (dev_handle);
	if (status)
	{
		cl_msg_out("cl_close_device: error closing device (%s)\n",
											strerror(errno));
	}
	return;
}

cl_status_t
cl_ioctl_device(
	IN	cl_dev_handle_t	dev_handle,
	IN	uint32_t		command,
	IN	void			*p_buf,
	IN	uintn_t			buf_size,
	OUT uintn_t			*p_num_bytes_ret )
{
	cl_ioctl_info_t		ioctl_args;
	int					retval = 0;

	/*
	 * Fill up ioctl_args and issue a real ioctl
	 */
	ioctl_args.command			= command;
	ioctl_args.p_buf			= p_buf;
	ioctl_args.buf_size			= buf_size;
	ioctl_args.num_bytes_ret	= 0;			/* for now */
	ioctl_args.io_status		= CL_SUCCESS;	/* lets start here */

	retval = ioctl(dev_handle, command, &ioctl_args);

	if (retval != 0)
	{
		cl_msg_out("cl_ioctl_device: error (%s) issuing command (0x%x)\n",
									strerror(errno), command);
		return CL_ERROR;
	}

	/*
	 * Set the Number of bytes returned from the Kernel.
	 * The driver sets the number of bytes returned in
	 * ioctl_args.num_bytes_ret
	 */
	if (p_num_bytes_ret != NULL)
	{
		*p_num_bytes_ret = ioctl_args.num_bytes_ret;
	}

	/*
	 * Return the status received from the Kernel
	 */

	return (ioctl_args.io_status);
}
