/*-
 * Copyright (c) 2001 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include "plugutil.h"

/* /\* items that do nothing other than pixmap like something *\/ */
/* static void handle_nothing(pier_t *pier, param_t *param, int type) */
/* { */

/* } */

/* handle launcher items */
static void handle_launch( param_t *param, int type)
{
   char *cmd;

	OPTIONAL_PARAM(&param->subparams, "cmd", string, cmd, NULL);
	if (!cmd)
   {
		PWARN("cmd subparam is required for pier launch items");
		return;
	}

   action_exec( 0, cmd );
}

/* table of handler functions for item types */
static struct paramhndlr {
	char	*text;
	int	type;
	void	(*handler)(param_t *param, int type);
} handlers[] = {
	{"launch",	0,	   handle_launch},
};

/* read parameters to create piers */
void parseparams()
{
   param_t *param;
	int i, n;

	/* go through every subparam and handle it */
	SUBPARAMS_FOREACH(i, param, &plugin_this->params)
       {

          for (n = 0; n < sizeof(handlers) / sizeof(struct paramhndlr); n++)
          {
             if( strcmp(handlers[n].text, param->value) == 0 )
             {
                handlers[n].handler( param, handlers[n].type );
                break;
             }
          }
       }
}
