/*-
 * Copyright (c) 2001 Jordan DeLong
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef TRACE_H
#define TRACE_H

#ifndef NDEBUG

# define trace(str, args...)				\
	warnx("%s:%d: " str, __FILE__, __LINE__ , ## args)

# define log( level, str, args...) \
   { \
      int i; \
      if( level > 0 ) \
      { \
        for( i = 0; i < level; i++ ) \
         {  \
           fprintf( stdout, " " ); \
         } \
   	   fprintf( stdout, " + [%s,%s,%d] " str "\n",  __FILE__, __FUNCTION__, __LINE__ , ## args); \
      } \
   }

# define log_enter(  str, args...) \
   { \
 	   fprintf( stdout, "-> [%s,%s] " str "\n", __FILE__, __FUNCTION__, ## args); \
   }

# define log_exit(  str, args...) \
   { \
 	   fprintf( stdout, "<- [%s,%s] " str "\n", __FILE__, __FUNCTION__, ## args); \
   }

#else

# define trace(ignore...)	((void) 0)
# define log(ignore...)	((void) 0)
# define log_enter(  str, args...) 	((void) 0)
# define log_exit(  str, args...) 	((void) 0)

#endif

#endif
